// HC11/CS8900 ethernet IP stack
//
// $Id: arp.c,v 2.1 2001/12/25 23:05:45 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"

#include "cs8900.h"
#include "ip.h"
#include "serial.h"

#define ARPBUFFERLEN (sizeof(struct ether_header) + sizeof(struct ether_arp))

//-----------------------------------------------------------------
// Variables
static u_int8_t arpBuffer[ARPBUFFERLEN];
u_int8_t arpMAC[ETHER_ADDR_LEN];
u_int8_t arpIP[sizeof(struct in_addr)];
u_int8_t arpTimeout;
u_int8_t arpValid;

//-----------------------------------------------------------------
void
arp_init()
{
    arpValid = 0;
}

//-----------------------------------------------------------------
// ARP packet generation
void
arp_send(const unsigned short int aOperation, const u_int8_t* aDestMAC, const u_int8_t* aDestIP)
{
   struct ether_header* ethHdr = (struct ether_header*) arpBuffer;
   struct ether_arp* arpPkt = (struct ether_arp*) (arpBuffer + sizeof(struct ether_header));

   // can't send ARP if our address is not set
   if (bootpPending == 0)
   {
      serial_print_debug('a');

      // build MAC header
      ethHdr->ether_type = ETHERTYPE_ARP;
   
      // build ARP packet header
      arpPkt->ea_hdr.ar_hrd = ARPHRD_ETHER;
      arpPkt->ea_hdr.ar_pro = ETHERTYPE_IP;
      arpPkt->ea_hdr.ar_hln = ETHER_ADDR_LEN;
      arpPkt->ea_hdr.ar_pln = sizeof(struct in_addr);
      arpPkt->ea_hdr.ar_op = aOperation;

      switch(aOperation)
      {
         case ARPOP_REPLY :
         {
            serial_print_debug('1');

            // build arp reply
	    memcpy(ethHdr->ether_dhost, aDestMAC, ETHER_ADDR_LEN);
            memcpy(arpPkt->arp_sha, ourMAC, ETHER_ADDR_LEN);
            memcpy(arpPkt->arp_tha, aDestMAC, ETHER_ADDR_LEN);
            memcpy(arpPkt->arp_spa, ourIP, sizeof(struct in_addr));
            memcpy(arpPkt->arp_tpa, aDestIP, sizeof(struct in_addr));
            break;
         }
         case ARPOP_REQUEST :
         {
            serial_print_debug('2');

            // build arp request
	    memset(ethHdr->ether_dhost, 0xff, ETHER_ADDR_LEN);
            memcpy(arpPkt->arp_sha, ourMAC, ETHER_ADDR_LEN);
	    memset(arpPkt->arp_tha, 0xff, ETHER_ADDR_LEN);
            memcpy(arpPkt->arp_spa, ourIP, sizeof(struct in_addr));
            memcpy(arpPkt->arp_tpa, aDestIP, sizeof(struct in_addr));

            // update table
            arpValid = 0;
            memcpy(arpMAC, aDestMAC, ETHER_ADDR_LEN);
            memcpy(arpIP, aDestIP, sizeof(struct in_addr));

            break;
         }
      }
    
      // send request/reply
      cs8900_send(arpBuffer, sizeof(struct ether_header) + sizeof(struct ether_arp));
   }
}

//-----------------------------------------------------------------
// ARP packet receive & action
void
arp_receive()
{
   const struct ether_arp* arpPkt = (struct ether_arp*) (rxBuffer + sizeof(struct ether_header));

   serial_print_debug('A');

   if (arpPkt->ea_hdr.ar_hrd == ARPHRD_ETHER && arpPkt->ea_hdr.ar_pro == ETHERTYPE_IP)
   {

      serial_print_debug('1');
      switch(arpPkt->ea_hdr.ar_op)
      {
	 case ARPOP_REQUEST:
	 {
#ifdef ARPDEBUG
	    serial_print_crlf();
	    serial_print_string("Got ARP request for ");
	    serial_print_int(arpPkt->arp_tpa[0]);
	    cpu_serial_send('.');
	    serial_print_int(arpPkt->arp_tpa[1]);
	    cpu_serial_send('.');
	    serial_print_int(arpPkt->arp_tpa[2]);
	    cpu_serial_send('.');
	    serial_print_int(arpPkt->arp_tpa[3]);
	    serial_print_crlf();
#endif
	    serial_print_debug('2');

	    // respond to requests for our ip address
	    if (!memcmp(ourIP, arpPkt->arp_tpa, sizeof(struct in_addr)))
	    {
	       serial_print_debug('3');
	       arp_send(ARPOP_REPLY, arpPkt->arp_sha, arpPkt->arp_spa);
	    }
	    break;
	 }
	 case ARPOP_REPLY:
	 {
#ifdef ARPDEBUG
            int i = 0;

	    serial_print_crlf();
	    serial_print_string("Got ARP reply for ");
	    serial_print_int(arpPkt->arp_spa[0]);
	    cpu_serial_send('.');
	    serial_print_int(arpPkt->arp_spa[1]);
	    cpu_serial_send('.');
	    serial_print_int(arpPkt->arp_spa[2]);
	    cpu_serial_send('.');
	    serial_print_int(arpPkt->arp_spa[3]);
	    serial_print_string(" MAC address is ");
            for(i = 0; i < 5; i++)
            {
               serial_print_hex_byte((u_int16_t) arpPkt->arp_sha[i], 0);
               cpu_serial_send(':');
            }
            serial_print_hex_byte((u_int16_t) arpPkt->arp_sha[5], 0);
	    serial_print_crlf();

	    serial_print_int(arpIP[0]);
	    cpu_serial_send('.');
	    serial_print_int(arpIP[1]);
	    cpu_serial_send('.');
	    serial_print_int(arpIP[2]);
	    cpu_serial_send('.');
	    serial_print_int(arpIP[3]);
	    serial_print_crlf();
#endif
	    serial_print_debug('3');

            // check this the correct response
            if (!memcmp(arpPkt->arp_spa, arpIP, sizeof(struct in_addr)))
            {
               // update table
               memcpy(arpMAC, arpPkt->arp_sha, ETHER_ADDR_LEN);
               arpValid = 1;
               serial_print_debug('4');
            }
	    break;
	 }
      }
   }
}

