// HC11/CS8900 ethernet IP stack
//
// $Id: cs8900.c,v 1.2 2001/12/25 23:07:27 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#define MACTEMPLATE    "Tom(0)"

#include "ether.h"
#include "cs8900.h"
#include "cs8900reg.h"
#include "serial.h"
#include "arp.h"
#include "ip.h"

//-----------------------------------------------------------------
// Globals
u_int8_t txBuffer[BUFFERLEN];
u_int8_t rxBuffer[BUFFERLEN];
u_int8_t ourMAC[ETHER_ADDR_LEN];


//-----------------------------------------------------------------
// cs8900 library routines
//
void
cs8900_init(u_int8_t aNodeId)
{
   u_int8_t i = 0;

   // MAC address
   memcpy(ourMAC, MACTEMPLATE, ETHER_ADDR_LEN);
   if (aNodeId > 9)
   {
      ourMAC[4] = aNodeId + 'a' - 10;
   }
   else
   {
      ourMAC[4] = aNodeId + '0';
   }

   serial_print_string("             - set MAC Address to ");
   for(i = 0; i < 5; i++)
   {
      serial_print_hex_byte((u_int16_t) ourMAC[i], 0);
      cpu_serial_send(':');
   }
   serial_print_hex_byte((u_int16_t) ourMAC[5], 0);
   serial_print_crlf();

   while(!cpu_cs8900_pp_reg_read(PP_SelfST) & INIT_DONE)
   {
      cpu_serial_send('.');
   }

   // enable full duplex
   cpu_cs8900_pp_reg_write(PP_TestCTL, TESTCTL_FDX);

   // accept addressed and broadcast valid frames 
   cpu_cs8900_pp_reg_write(PP_RxCTL, RX_OK_ACCEPT     |
		       RX_IA_ACCEPT               |
		       RX_BROADCAST_ACCEPT);

   // set our MAC address
   cpu_cs8900_pp_reg_write(PP_IA,   ourMAC[1] << 8 | ourMAC[0]);
   cpu_cs8900_pp_reg_write(PP_IA+2, ourMAC[3] << 8 | ourMAC[2]);
   cpu_cs8900_pp_reg_write(PP_IA+4, ourMAC[5] << 8 | ourMAC[4]);

   // enable receiver and transmitter, force interface to 10baseT
   cpu_cs8900_pp_reg_write(PP_LineCTL, SERIAL_RX_ON | SERIAL_TX_ON);
}

//-----------------------------------------------------------------
int
cs8900_check()
{
   u_int16_t manufId = 0;
   u_int16_t prodId = 0;
   u_int8_t  retVal = 0;

   manufId = cpu_cs8900_pp_reg_read(PP_ChipID);
   if (manufId == 0x630e)
   {
      prodId = cpu_cs8900_pp_reg_read(PP_ChipID + 2);

      serial_print_string("Found CS8900 - manufacturer ID 0x");
      serial_print_hex_int(manufId, 1);
      serial_print_crlf();
      serial_print_string("             - product ID 0x");
      serial_print_hex_int(prodId, 1);
      serial_print_crlf();

      retVal = 1;
   }
   
   return retVal;
}

//-----------------------------------------------------------------
void
cs8900_send(u_int8_t* aBuffer, u_int16_t aLength)
{
   register u_int16_t i = 0;
   struct ether_header* ethHdr = (struct ether_header*) aBuffer;
   u_int8_t* txPtr = aBuffer;

   serial_print_debug('s');

   // insert our MAC address
   memcpy((void *) ethHdr->ether_shost, (void *) ourMAC, ETHER_ADDR_LEN);
   
   // Initialise tx command register
   // start transmission only when all bytes are available
   cpu_cs8900_reg_write(CS8900_TxCMD_MSB, 0);
   cpu_cs8900_reg_write(CS8900_TxCMD_LSB, 0xc0);

#ifdef TXDEBUG
   serial_print_crlf();
   serial_print_string("Sending ");
   serial_print_int(aLength);
   serial_print_crlf();
#endif

   // write packet length
   cpu_cs8900_reg_write(CS8900_TxLength_MSB, (aLength >> 8) & 0xff);
   cpu_cs8900_reg_write(CS8900_TxLength_LSB, aLength & 0xff);

   // wait for packet space : TODO - TIMEOUT
   while (!(cpu_cs8900_pp_reg_read(PP_BusST) & READY_FOR_TX_NOW)) {};

   // pad odd packets
   if (aLength % 2)
   {
      *((u_int8_t *) aBuffer + aLength) = 0;
   }

   // write to xmit buffer (convert length to words & round odd bytes up)
   aLength = (aLength + 1) >> 1;
   for (i = 1; i <= aLength; i++)
   {
#ifdef TXDEBUG
      serial_print_hex_byte(*txPtr, 1);
      cpu_serial_send(' ');
#endif

       cpu_cs8900_reg_write(CS8900_DATA_PORT0_LSB, *txPtr++);

#ifdef TXDEBUG
      serial_print_hex_byte(*txPtr, 1);
      cpu_serial_send(' ');
      if (i % 8 == 0) serial_print_crlf();
#endif

       cpu_cs8900_reg_write(CS8900_DATA_PORT0_MSB, *txPtr++);
   }
   
#ifdef TXDEBUG
   serial_print_crlf();
#endif
}

//-----------------------------------------------------------------
void
cs8900_receive()
{
   u_int16_t rxStatus = 0;
   u_int16_t rxLength = 0;
   register u_int16_t i = 0;
   u_int8_t store = 1;
   register u_int8_t thisbyte = 0;
   struct ether_header *ethHdr = (struct ether_header *) rxBuffer;

   serial_print_debug('R');

   // read length and status
   rxStatus = (cpu_cs8900_reg_read(CS8900_DATA_PORT0_MSB) << 8) | cpu_cs8900_reg_read(CS8900_DATA_PORT0_LSB);
   rxLength = (cpu_cs8900_reg_read(CS8900_DATA_PORT0_MSB) << 8) | cpu_cs8900_reg_read(CS8900_DATA_PORT0_LSB);

#ifdef RXDEBUG
   serial_print_crlf();
   serial_print_string("Receive length = ");
   serial_print_int(rxLength);
   serial_print_crlf();
#endif
   // discard long packets
   if (rxLength > BUFFERLEN) store = 0;

   // read packet in words
   for(i = 0; i < rxLength;i += 2)
   {
#ifdef RXDEBUG
      if (i % 16 == 0 && i != 0) serial_print_crlf();
#endif

      thisbyte = cpu_cs8900_reg_read(CS8900_DATA_PORT0_LSB);
      if (store)
      {
	 rxBuffer[i] = thisbyte;
      }

#ifdef RXDEBUG
      serial_print_hex_byte(thisbyte, 1);
      cpu_serial_send(' ');
#endif
      thisbyte = cpu_cs8900_reg_read(CS8900_DATA_PORT0_MSB);
      if (store)
      {
	 rxBuffer[i+1]   = thisbyte;
      }
#ifdef RXDEBUG
      serial_print_hex_byte(thisbyte, 1);
      cpu_serial_send(' ');
#endif
   }
#ifdef RXDEBUG
   serial_print_crlf();
#endif

   // analyse packet
   switch(ethHdr->ether_type)
   {
      case ETHERTYPE_ARP:
      {
	 serial_print_debug('1');
	 arp_receive();
	 break;
      }
      case ETHERTYPE_IP:
      {
	 serial_print_debug('2');
         ip_receive();
	 break;
      }
   }
#ifdef RXDEBUG
   serial_print_crlf();
#endif
#ifdef DEBUG
   serial_print_crlf();
#endif
}

//-----------------------------------------------------------------
// poll the cs8900 for received packets
void 
cs8900_poll()
{
   u_int16_t rxEvent = cpu_cs8900_pp_reg_read(PP_RxEvent);
   
   if (rxEvent & RX_OK)
   {
      cs8900_receive();
   }
}

//-----------------------------------------------------------------
// dump a buffer
void
cs8900_dump(const u_int8_t* aBuffer, u_int16_t aLength)
{
   u_int16_t i = 0;

   serial_print_crlf();
	       
   for (i = 0; i < aLength;)
   {
      serial_print_hex_byte(aBuffer[i], 1);
      cpu_serial_send(' ');
      if (++i % 16 == 0) serial_print_crlf();
   }

   serial_print_crlf();
}
