// HC11/CS8900 ethernet IP stack
//
// $Id: ether.c,v 2.1 2001/12/25 23:06:29 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#define EXTERN

#include "ether.h"

#include "cs8900.h"
#include "cpu.h"
#include "udp.h"
#include "tcp.h"
#include "ip.h"
#include "arp.h"
#include "serial.h"
#include "timer.h"
#include "bootp.h"


//-----------------------------------------------------------------
int
main()
{
   u_int8_t nodeId = 0;

   cpu_init();
   time_init();
   ip_init();
#ifdef TCP
   tcp_init();
#endif

   nodeId = cpu_get_nodeid();

   serial_print_string("\r\nCS8900 IP (node = ");
   serial_print_hex_byte(nodeId, 0);
   serial_print_string(") $Revision: 2.1 $ - ");
   serial_print_string(__DATE__ " " __TIME__ "\r\n\r\n");

   while (!cs8900_check())
   {
      serial_print_string("Failed to find CS8900\n\r");
   }

   cs8900_init(nodeId);


#ifdef BOOTP
   // flag bootp as pending
   bootpPending = 1;
   bootpTimeout = 1;
   serial_print_string("\r\nBootp .");
#else
   // not using bootp so fix our IP address
   ourIP[0] = IPADDRESS_0;
   ourIP[1] = IPADDRESS_1;
   ourIP[2] = IPADDRESS_2;
   ourIP[3] = 200 + nodeId;

   serial_print_string("             - fixed ip address is ");
   serial_print_int(ourIP[0]);
   cpu_serial_send('.');
   serial_print_int(ourIP[1]);
   cpu_serial_send('.');
   serial_print_int(ourIP[2]);
   cpu_serial_send('.');
   serial_print_int(ourIP[3]);
   serial_print_crlf();

   // class C (a.b.c.255)
   ourBCAST[0] = IPADDRESS_0;
   ourBCAST[1] = IPADDRESS_1;
   ourBCAST[2] = IPADDRESS_2;
   ourBCAST[3] = 255;
   serial_print_string("             - broadcast address is ");
   serial_print_int(ourBCAST[0]);
   cpu_serial_send('.');
   serial_print_int(ourBCAST[1]);
   cpu_serial_send('.');
   serial_print_int(ourBCAST[2]);
   cpu_serial_send('.');
   serial_print_int(ourBCAST[3]);
   serial_print_crlf();
#endif

   // wait for rx event
   while (1)
   {
#ifdef BOOTP
      if (bootpPending && bootpTimeout)
      {
	 // send bootp packet
	 udp_bootp_send();
      }
#endif

// This is an example of a board initiated transmit that will generate ARP requests etc.
#if 0
      if (!bootpPending)
      {
         const u_int8_t destIP[sizeof(struct in_addr)] = {192, 168, 0, 1};
         
         udp_send(IPPORT_DISCARD, IPPORT_DISCARD, destIP, sizeof(struct bootp));
      }
#endif
      
      // check for received packet
      cs8900_poll();
   }

   return(0);
}


