// HC11/CS8900 ethernet IP stack
//
// $Id: icmp.c,v 1.2 2001/12/25 23:07:27 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"

#include "ip.h"
#include "serial.h"
#include "cs8900.h"

//-----------------------------------------------------------------
// send an ICMP echo response using the received request data
static void
icmp_send_echoreply()
{
   const struct ip* rxIpPkt = (struct ip*) (rxBuffer + sizeof(struct ether_header));
   const struct icmphdr* rxIcmpPkt = (struct icmphdr*) (rxBuffer + sizeof(struct ether_header) + sizeof(struct ip));
   struct ip* ipPkt = (struct ip*) (txBuffer + sizeof(struct ether_header));
   struct icmphdr* icmpPkt = (struct icmphdr*) (txBuffer + sizeof(struct ether_header) + sizeof(struct ip));
   const u_int16_t* mem = (u_int16_t*) icmpPkt;
   u_int16_t icmpPktLen = ntohs(rxIpPkt->ip_len) - sizeof(struct ip);
   u_int32_t newcksum = 0;
   register u_int16_t i = 0;

   serial_print_debug('p');

   // create reply
   icmpPkt->type = ICMP_ECHOREPLY;
   icmpPkt->code = 0;
   icmpPkt->un.echo.id = rxIcmpPkt->un.echo.id;
   icmpPkt->un.echo.sequence = rxIcmpPkt->un.echo.sequence;

   // copy payload
   memcpy((u_int8_t*) icmpPkt + sizeof(struct icmphdr),
	  (u_int8_t*) rxIcmpPkt + sizeof(struct icmphdr),
	  icmpPktLen - sizeof(struct icmphdr));

   // checksum
   icmpPkt->checksum = 0;

   // pad odd packets
   if (icmpPktLen % 2)
   {
      *((u_int8_t *) mem + icmpPktLen) = 0;
   }

   // checksum (convert length to words & round odd bytes up)
   icmpPktLen = (icmpPktLen + 1) >> 1;
   for(i = 0; i < icmpPktLen; i++, mem++)
   {
      newcksum += ntohs(*mem);
   }
   icmpPkt->checksum = htons((u_int16_t) ~((newcksum >> 16) + (newcksum & 0xffff)));
   
   // replace destination IP with source of rx packet
   memcpy(&ipPkt->ip_dst, &rxIpPkt->ip_src, sizeof(struct in_addr));
   
   // packet length, protocol & id are the same
   ipPkt->ip_len = rxIpPkt->ip_len;
   ipPkt->ip_p = IPPROTO_ICMP;
   ipPkt->ip_id = rxIpPkt->ip_id;
   ipPkt->ip_ttl = rxIpPkt->ip_ttl;

   ip_send();
}

//-----------------------------------------------------------------
// ICMP packet receive & action
void
icmp_receive(const uint16_t aLength)
{
   const struct icmphdr* icmpPkt = (struct icmphdr*) (rxBuffer + sizeof(struct ether_header) + sizeof(struct ip));
   const u_int16_t* mem = (u_int16_t*) icmpPkt;
   u_int32_t newcksum = 0;
   register u_int16_t i = 0;
   u_int16_t length = aLength;

   serial_print_debug('P');

   // pad odd packets
   if (length % 2)
   {
      *((u_int8_t *) mem + length) = 0;
   }

   // checksum (convert length to words & round odd bytes up)
   length = (length + 1) >> 1;
   for (i = 0; i < length; i++, mem++)
   {
      // skip existing checksum
      if (i != 1)
      {
	 newcksum += ntohs(*mem);
      }
   }

   newcksum = ~((newcksum >> 16) + (newcksum & 0xffff));
   if ((newcksum & 0xffff) == ntohs(icmpPkt->checksum))
   {
      switch(icmpPkt->type)
      {
	 case ICMP_ECHO:
	    serial_print_debug('1');
	    icmp_send_echoreply();
	    break;
      }
   }
   else
   {
      serial_print_string("ICMP checksum error 0x");
      serial_print_hex_int(newcksum,1);
      serial_print_string(" != 0x");
      serial_print_hex_int(ntohs(icmpPkt->checksum), 1);
      serial_print_crlf();
      cs8900_dump(rxBuffer, BUFFERLEN);
   }
}

