// includes for the hc11f1
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#ifndef _M68HC11_PORTS_DEF_F1_H
#define _M68HC11_PORTS_DEF_F1_H

/* 68HC11F1 register address offsets (range 0x40..0x5F)
   The absolute address of the I/O register depends on the setting
   of the M6811_INIT register.  At init time, the I/O registers are
   mapped at 0x1000.  Address of registers is then:

   0x1000 + M6811F1_xxx
*/
#define M6811F1_PORTG	0x02	/* IO port G */
#define M6811F1_DDRG	0x03	/* DDR port G */
#define M6811F1_CSSTRH	0x5c	/* chip select clock stretch */ 
#define M6811F1_CSCTL	0x5d	/* chip select control */ 
#define M6811F1_CSGADR	0x5e	/* chip select address register */ 
#define M6811F1_CSGSIZ	0x5f	/* chip select size control */ 

/* Flags of the CSSTRH  register */
#define M6811F1_IO1SA   0x80    /* I/O CS1 stretch select A */
#define M6811F1_IO1SB   0x40    /* I/O CS1 stretch select B */
#define M6811F1_IO2SA   0x20    /* I/O CS2 stretch select A */
#define M6811F1_IO2SB   0x10    /* I/O CS2 stretch select B */
#define M6811F1_GSTHA   0x08    /* General purpose CS stretch select A */
#define M6811F1_GSTHB   0x04    /* General purpose CS stretch select B */
#define M6811F1_PSTHA   0x02    /* Program CS stretch select A */
#define M6811F1_PSTHB   0x01    /* Program CS stretch select B */

/* Flags of the CSCTL register */
#define M6811F1_IO1EN   0x80    /* I/O CS1 enable */
#define M6811F1_IO1PL   0x40    /* I/O CS1 polarity */
#define M6811F1_IO2EN   0x20    /* I/O CS2 enable */
#define M6811F1_IO2PL   0x10    /* I/O CS2 polarity */
#define M6811F1_GCSPR   0x08    /* General purpose CS priority */
#define M6811F1_PCSEN   0x04    /* Program CS enable (R/O) */
#define M6811F1_PSIZA   0x02    /* Program CS size A */
#define M6811F1_PSIZB   0x01    /* Program CS size B */

/* Flags of the CSGSIZ register */
#define M6811F1_IO1AV   0x80    /* I/O CS1 address valid */ 
#define M6811F1_IO2AV   0x40    /* I/O CS2 address valid */
#define M6811F1_GNPOL   0x10    /* General purpose CS polarity */
#define M6811F1_GAVLD   0x08    /* General purpose CS address valid */
#define M6811F1_GSIZA   0x04    /* General purpose CS size A */
#define M6811F1_GSIZB   0x02    /* General purpose CS size B */
#define M6811F1_GSIZC   0x01    /* General purpose CS size C */

/* F1 has 0xFFF bytes of ports */
#undef  M6811_IO_SIZE
#define M6811_IO_SIZE   (0xFFF)

/* The I/O registers are represented by a volatile array.
   Address if fixed at link time.  */
extern volatile unsigned char _io_ports[M6811_IO_SIZE];

#endif /* _M68HC11F1_PORTS_DEF_H */

