// HC11/CS8900 ethernet IP stack
//
// $Id: serial.c,v 1.1 2001/11/04 09:55:02 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"
#include "cpu.h"

//-----------------------------------------------------------------
void serial_print_debug(u_int8_t c)
{
#ifdef DEBUG
   cpu_serial_send(c);
#endif
}

//-----------------------------------------------------------------
void
serial_print_crlf()
{
   cpu_serial_send('\r');
   cpu_serial_send('\n');
}

//-----------------------------------------------------------------
void 
serial_print_string(char* aString)
{
   while(*aString != '\0')
   {
      cpu_serial_send(*aString);
      aString++;
   }
}

//-----------------------------------------------------------------
void
serial_print_int(u_int16_t aNumber)
{
   u_int16_t divisor = 10000;
   u_int16_t modulo = 0;
   u_int8_t firstDigit = 1;

   if (aNumber == 0)
   {
      cpu_serial_send('0');
   }
   else while (divisor > 0)
   {
      modulo = aNumber - (aNumber % divisor);
      if (modulo > 0 || !firstDigit)
      {
	 firstDigit = 0;
	 cpu_serial_send('0' + modulo / divisor);
	 aNumber -= modulo;
      }
      
      divisor /= 10;
   }
}

//-----------------------------------------------------------------
void
serial_print_hex_byte(u_int8_t aNumber, u_int8_t aLeadingZeros)
{
   u_int8_t divisor = 0x10;
   u_int8_t value = 0;
   u_int8_t modulo = 0;
   u_int8_t firstChar = 1;

   if (aNumber == 0)
   {
      if (aLeadingZeros) serial_print_string("00"); else cpu_serial_send('0');
   }
   else while (divisor > 0)
   {
      modulo = aNumber - (aNumber % divisor);
      value =  modulo / divisor;
      if (value != 0 || aLeadingZeros || !firstChar)
      {
	 if (value != 0) firstChar = 0;
	 if (value < 10)
	 {
	    cpu_serial_send('0' + value);
	 }
	 else 
	 {
	    cpu_serial_send('a' - 10 + value);
	 }
      }
      aNumber -= modulo;
      divisor /= 0x10;
   }
}

//-----------------------------------------------------------------
void
serial_print_hex_int(u_int16_t aNumber, u_int8_t aLeadingZeros)
{
   u_int16_t  divisor = 0x1000;
   u_int8_t   value = 0;
   u_int16_t  modulo = 0;
   u_int8_t   firstChar = 1;

   if (aNumber == 0)
   {
      if (aLeadingZeros) serial_print_string("0000"); else cpu_serial_send('0');
   }
   else while (divisor > 0)
   {
      modulo = aNumber - (aNumber % divisor);
      value =  modulo / divisor;
#if 0
      serial_print_crlf();
      serial_print_hex_byte(value, 1);
      serial_print_crlf();
#endif

      if (value != 0 || aLeadingZeros || !firstChar)
      {
	 if (value != 0) firstChar = 0;
	 if (value < 10)
	 {
	    cpu_serial_send('0' + value);
	 }
	 else 
	 {
	    cpu_serial_send('a' - 10 + value);
	 }
      }
      aNumber -= modulo;
      divisor /= 0x10;
   }
}

