// HC11/CS8900 ethernet IP stack
//
// $Id: tcp.c,v 2.0 2001/11/12 12:34:20 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#define TCPMAXCONN     10

#include "ether.h"

#include "ip.h"
#include "serial.h"
#include "http.h"
#include "cs8900.h"
#include "arp.h"
#include "tcpstruct.h"

//-----------------------------------------------------------------
// Variables
u_int32_t tcpStartSeq;
static tcpConn tcpTable[TCPMAXCONN];        

//-----------------------------------------------------------------
// dump contents of tcp packet
void tcp_dump(const struct ip* packet)
{
   struct tcphdr* tcpPkt = (struct tcphdr*) ((uint8_t *) packet + sizeof(struct ip));

   serial_print_string("\r\nTCP Packet dump\r\n  source ip = 0x");
   serial_print_hex_int(ntohl(packet->ip_src.s_addr) >> 16, 1);
   serial_print_hex_int(ntohl(packet->ip_src.s_addr) & 0xffff, 1);

   serial_print_string("\r\n  dest ip = 0x");
   serial_print_hex_int(ntohl(packet->ip_dst.s_addr) >> 16, 1);
   serial_print_hex_int(ntohl(packet->ip_dst.s_addr) & 0xffff, 1);

   serial_print_string("\r\n  source port = ");
   serial_print_int(ntohs(tcpPkt->source));

   serial_print_string("\r\n  dest port = ");
   serial_print_int(ntohs(tcpPkt->dest));

   serial_print_string("\r\n  seq = 0x");
   serial_print_hex_int(ntohl(tcpPkt->seq) >> 16, 1);
   serial_print_hex_int(ntohl(tcpPkt->seq) & 0xffff, 1);

   serial_print_string("\r\n  ack seq = 0x");
   serial_print_hex_int(ntohl(tcpPkt->ack_seq) >> 16, 1);
   serial_print_hex_int(ntohl(tcpPkt->ack_seq) & 0xffff, 1);

   serial_print_string("\r\n  tcp header length (bytes) = ");
   serial_print_int(tcpPkt->doff * 4);
   
   serial_print_string("\r\n  flags = ");
   if (tcpPkt->fin) cpu_serial_send('F');
   if (tcpPkt->syn) cpu_serial_send('S');
   if (tcpPkt->rst) cpu_serial_send('R');
   if (tcpPkt->psh) cpu_serial_send('P');
   if (tcpPkt->ack) cpu_serial_send('A');
   if (tcpPkt->urg) cpu_serial_send('U');

   serial_print_string("\r\n  window = ");
   serial_print_int(ntohs(tcpPkt->window));

   serial_print_string("\r\n  check = 0x");
   serial_print_hex_int(ntohs(tcpPkt->check), 1);

   serial_print_string("\r\n  urgent pointer = 0x");
   serial_print_hex_int(ntohs(tcpPkt->urg_ptr), 1);

   serial_print_crlf();
}

//-----------------------------------------------------------------
// send a tcp packet given the connection, flags and payload length
void
tcp_send(tcpConn* aConn,
	 u_int8_t flags,
	 const u_int16_t aLength)
{
   struct ip* ipPkt = (struct ip*) (rxBuffer + sizeof(struct ether_header));
   struct tcphdr* tcpPkt = (struct tcphdr*) (rxBuffer + sizeof(struct ether_header) + sizeof(struct ip));

   const u_int16_t* mem = (u_int16_t*) &(ipPkt->ip_src);
   u_int16_t length = aLength + sizeof(struct tcphdr) + sizeof(u_int32_t);
   u_int32_t newcksum = IPPROTO_TCP + length;
   register u_int16_t i = 0;

   serial_print_debug('t');

   // TCP header
   tcpPkt->source = aConn->ourPort;
   tcpPkt->dest = aConn->farPort;
   tcpPkt->seq = aConn->ourSeq;
   if (flags & TCP_FLAG_ACK)
   {
      tcpPkt->ack_seq = htonl(ntohl(aConn->farSeq) + 1);
      tcpPkt->ack = 1;
   }
   else
   {
      tcpPkt->ack_seq = 0;
      tcpPkt->ack = 0;
   }
   tcpPkt->doff = sizeof(struct tcphdr) / sizeof(u_int32_t) + 1; 
   tcpPkt->fin = (flags & TCP_FLAG_FIN)?1:0;
   tcpPkt->syn = (flags & TCP_FLAG_SYN)?1:0;
   tcpPkt->rst = (flags & TCP_FLAG_RST)?1:0;
   tcpPkt->psh = (flags & TCP_FLAG_PSH)?1:0;
   tcpPkt->urg = (flags & TCP_FLAG_URG)?1:0;
   tcpPkt->window = 1;
   tcpPkt->check = 0;
   tcpPkt->urg_ptr = 0;

   // MSS option (max length is buffer length without MAC/IP/TCP + MSS)
   *(u_int32_t *) ((u_int8_t *) tcpPkt + sizeof(struct tcphdr)) = 
      ntohl(0x02040000 | (BUFFERLEN - sizeof(struct ether_header) - sizeof(struct ip) - sizeof(struct tcphdr) - sizeof(u_int32_t)));

   // IP header
   ipPkt->ip_dst.s_addr = aConn->farAddr;
   memcpy(&ipPkt->ip_src, ourIP, sizeof(struct in_addr));
   ipPkt->ip_len = htons(aLength + sizeof(struct ip) + sizeof(struct tcphdr));
   ipPkt->ip_p = IPPROTO_TCP;
   ipPkt->ip_id = 0;
   ipPkt->ip_ttl = 64;

   // add IP source & destination
   newcksum += ntohs(*mem++);
   newcksum += ntohs(*mem++);
   newcksum += ntohs(*mem++);
   newcksum += ntohs(*mem);

   // packet checksum
   mem = (u_int16_t*) tcpPkt;
   
   // pad odd packets
   if (length % 2)
   {
      *((u_int8_t *) mem + length) = 0;
   }

   // calc checksum (convert length to words & round odd bytes up)
   length = (length + 1) >> 1;
   for (i = 0; i < length; i++, mem++)
   {
      newcksum += ntohs(*mem);
   }

   // insert checksum & send
   tcpPkt->check = htons((u_int16_t) ~((newcksum >> 16) + (newcksum & 0xffff)));

#ifdef DEBUG   
   tcp_dump(ipPkt);
#endif

   ip_send();
}

//-----------------------------------------------------------------
// Locate a connection entry in the table given params.  If no match is
// found, the first free connection is returned, if non is available
// then zero is returned.  All params are in network byte ordering.
tcpConn*
tcp_find_conn(const u_int32_t farAddr, const u_int16_t farPort, const u_int16_t ourPort)
{
   register u_int8_t i = 0;
   u_int8_t firstFree = TCPMAXCONN;
    
   for (i = 0; i < TCPMAXCONN; i++)
   {
      if (tcpTable[i].state == LISTEN)
      {
	 if (firstFree == TCPMAXCONN) firstFree = i;
      } 
      else if (tcpTable[i].farAddr == farAddr &&
	       tcpTable[i].farPort == farPort &&
	       tcpTable[i].ourPort == ourPort)
      {
	 return &tcpTable[i];
      }
   }
    
   if (firstFree == TCPMAXCONN)
   {
      return 0;
   }
   else
   {
      tcpConn* entry = &tcpTable[firstFree];
      entry->farAddr = farAddr;
      entry->farPort = farPort;
      entry->ourPort = ourPort;

      return entry;
   }
}

//-----------------------------------------------------------------
// TCP packet receive & action
void
tcp_receive(const u_int16_t aLength)
{
   const struct tcphdr* tcpPkt = (struct tcphdr*) (rxBuffer + sizeof(struct ether_header) + sizeof(struct ip));
   const struct ip* ipPkt = (struct ip*) (rxBuffer + sizeof(struct ether_header));
   const u_int16_t* mem = (u_int16_t*) &(ipPkt->ip_src);
   u_int16_t length = aLength;
   u_int32_t newcksum = IPPROTO_TCP + length;
   register u_int16_t i = 0;

   serial_print_debug('T');

   // checksum includes protocol (see above), tcp length (see
   // above) and IP source & destination
   newcksum += ntohs(*mem++);
   newcksum += ntohs(*mem++);
   newcksum += ntohs(*mem++);
   newcksum += ntohs(*mem);

   // packet checksum
   mem = (u_int16_t*) tcpPkt;

   // pad odd packets
   if (length % 2)
   {
      *((u_int8_t *) mem + length) = 0;
   }

   // calc checksum (convert length to words & round odd bytes up)
   length = (length + 1) >> 1;
   for (i = 0; i < length; i++, mem++)
   {
      // skip existing checksum
      if (i != 8)
      {
	 newcksum += ntohs(*mem);
      }
   }
   
   newcksum = ~((newcksum >> 16) + (newcksum & 0xffff));
   if ((newcksum & 0xffff) != ntohs(tcpPkt->check))
   {
      serial_print_string("\r\nTCP checksum error 0x");
      serial_print_hex_int(newcksum, 1);
      serial_print_string(" != 0x");
      serial_print_hex_int(ntohs(tcpPkt->check), 1);
      serial_print_crlf();
      cs8900_rx_dump();
   }
   else
   {
      tcpConn* conn = tcp_find_conn(ipPkt->ip_src.s_addr, tcpPkt->source, tcpPkt->dest);

      tcp_dump(ipPkt);

      if (conn != 0)
      {
	 // state
	 switch(conn->state)
	 {
	    case LISTEN:
	       if (tcpPkt->syn)
	       {
		  serial_print_debug('1');
		  conn->state = SYN_RCVD;
		  conn->ourSeq = ntohl(tcpStartSeq);
		  conn->farSeq = tcpPkt->seq;
		  // send syn + ack
		  tcp_send(conn, TCP_FLAG_ACK | TCP_FLAG_SYN, 0);
	       }
	       break;
	    case SYN_RCVD:
	       if (tcpPkt->ack)
	       {
		  // TODO : check seq being ack'd
		  serial_print_debug('2');
		  serial_print_debug('+');
		  conn->state = ESTABLISHED;
		  conn->farSeq = tcpPkt->seq;
	       } else if (tcpPkt->syn)
	       {
		  // retransmit - TODO check seq 
		  serial_print_debug('2');
		  serial_print_debug('-');
		  conn->farSeq = tcpPkt->seq;
		  // send syn + ack
		  tcp_send(conn, TCP_FLAG_ACK | TCP_FLAG_SYN, 0);
	       }
	       break;
	    case ESTABLISHED:
	       if (tcpPkt->fin)
	       {
		  serial_print_debug('3');
		  conn->state = LAST_ACK;

		  // TODO : send ACK & FIN
	       }
	       break;
	    case LAST_ACK:
	       if (tcpPkt->ack)
	       {
		  // TODO : check seq being ack'd
		  serial_print_debug('4');
		  conn->state = LISTEN;
	       }
	       break;
	 }

	 // data
	 switch(tcpPkt->dest)
	 {
	    case 80: // HTTP
#ifdef HTTP
	       http_receive(conn, aLength);
#endif
	       break;
	    default:
	       break;
	 }
      }
      else
      {
	 serial_print_string("\r\nTCP maximum connections exceeded\r\n");
      }
   }
}

//-----------------------------------------------------------------
// initialise the tcp state machine for all connections
void 
tcp_init()
{
   u_int8_t i = 0;

   for(i=0; i < TCPMAXCONN; i++)
   {
      tcpTable[i].farAddr = 0;
      tcpTable[i].state = LISTEN;
      tcpTable[i].ourPort = 0;
      tcpTable[i].farPort = 0;
      tcpTable[i].ourSeq = 0;
      tcpTable[i].farSeq = 0;
   }
   
   tcpStartSeq = 0;
}

