// HC11/CS8900 ethernet IP stack
//
// $Id: timer.c,v 2.1 2001/12/25 23:08:14 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"
#include "cpu.h"
#ifdef TCP
#include "tcp.h"
#endif
#include "serial.h"
#include "ip.h"
#include "arp.h"

//-----------------------------------------------------------------
// Globals

u_int32_t currentTime;
u_int16_t timerTicks;


//-----------------------------------------------------------------
// Prototypes

void time_of_day(void) __attribute__((interrupt));

//-----------------------------------------------------------------
void
time_init()
{
    timerTicks = 0;
    currentTime = 0;
}

//-----------------------------------------------------------------
// Time of day ISR and maintenance
// 
// ISR will fire every 2.048 ms.  time_of_day is hooked to the RTI
// Real Time Interrupt in vectors.s for standalone or in hc11_init
// for buffalo rom
//
void
time_of_day()
{
   // fire every second
   if (++timerTicks == 1000000/16384)
   {
      timerTicks = 0;
      currentTime++;
#ifdef TCP
      tcpStartSeq++;
#endif
      serial_print_debug('!');

      // 1s timeout for bootp
#ifdef BOOTP
      if (bootpPending) bootpTimeout = BOOTPTIMEOUT;
#endif

      // decrement timeout for ARP
      if (arpTimeout) arpTimeout--;
   }

   cpu_enable_tod();
}
